/* Xmsg.c
 * 
 * version 0.3.4
 *
 * last changed : 12/05/2001
 *
 * currently displays the text you give as the first argument on the
 * cmdline
 * 
 * current action : code cleanup and configuration parameters
 * doesn't work yet
 */

#include <X11/Xlib.h>
#include <X11/xpm.h>
#include <X11/extensions/shape.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

/* project includes */

#include "config.h"

/* default values */

#define IMG_X 0
#define IMG_Y 0
#define IMG_W 50
#define IMG_H 50

#define MSG_X 50
#define MSG_Y 0
#define MSG_W 950
#define MSG_H 50

#define DEF_FONT "-*-lucida-bold-r-normal-*-24-*-*-*-*-*-*-*"

/* debug ? */
#define DEBUG

XFontStruct* font_info;       /* Font structure, used for drawing text.    */

/* create two windows, one for the icon and one for the message
 * put them next to each other
 * display the icon in the first
 * display the message graphically in the second
 */


Window 
CreateWindow (Display* dpy, int screen_num, 
              int x, int y, int width, int height)
{
    /* Create a window and map it */
    
    int vmask;
    XSetWindowAttributes attribs;
    Window w;

    /* message window */
    vmask = CWSaveUnder|CWOverrideRedirect|CWColormap|CWBorderPixel;
    attribs.save_under = True;
    attribs.override_redirect = True;
    attribs.colormap = DefaultColormap (dpy, screen_num);
    attribs.background_pixel = 1000;
    attribs.border_pixel = 0; /* do not care */

    w = XCreateWindow (dpy, DefaultRootWindow (dpy), x, y, width, height,
                       0, CopyFromParent, CopyFromParent, CopyFromParent, 
                       vmask, &attribs);

    XMapWindow (dpy, w);
    return w;
}

GC
CreateGC (Display* display, Window win, char color[])
{
  /* Create a graphics context with the given colo as foreground color */
  
  GC gc;                                /* handle of newly created GC.  */
  unsigned long valuemask = 0;          /* which values in 'values'  to */
                                        /* check when creating the GC.  */
                                        /* check when creating the GC.  */
  XGCValues values;                     /* initial values for the GC.   */
  unsigned int line_width = 2;          /* line width for the GC.       */
  int line_style = LineSolid;           /* style for lines drawing and  */
  int cap_style = CapButt;              /* style of the line's edje and */
  int join_style = JoinBevel;           /*  joined lines.               */
  int screen_num = DefaultScreen(display);

  Colormap colormap;
  XColor col;
  
  gc = XCreateGC(display, win, valuemask, &values);
  if (gc < 0) {
        fprintf(stderr, "XCreateGC: \n");
  }
  XSetSubwindowMode ( display, gc, IncludeInferiors );
  
  /* allocate foreground and background colors for this GC. */
  colormap = DefaultColormap (display, 0);
  XParseColor (display, colormap, color, &col);
  XAllocColor (display, colormap, &col);

  XSetForeground(display, gc, col.pixel);
  XSetBackground(display, gc, BlackPixel(display, screen_num));
  
  /* define the style of lines that will be drawn using this GC. */
  XSetLineAttributes(display, gc,
                     line_width, line_style, cap_style, join_style);
  
  /* define the fill style for the GC. to be 'solid filling'. */
  XSetFillStyle(display, gc, FillSolid);

  return gc;
}

int 
DrawText (Display* display, Window window, XFontStruct* font_info, 
           char *text, GC gc_fg, GC gc_bo)
{
  /* display the text using a shadowy effect */

  Pixmap shapemask;
  GC gc_shape;
  XGCValues gcv;

  gcv.graphics_exposures = False;
  gcv.function = GXcopy;
  gcv.foreground = WhitePixel (display, 0);
  gcv.subwindow_mode = IncludeInferiors;

  shapemask = XCreatePixmap (display, window, MSG_W, MSG_H, 1);
  gc_shape = XCreateGC (display, shapemask, 0 , &gcv);
  XSetFont (display, gc_shape, font_info->fid);
  XSetForeground (display, gc_shape, 0);
  XFillRectangle (display, shapemask, gc_shape, 0, 0, MSG_W, MSG_H);
  XSetForeground (display, gc_shape, 1);

  
  {
    /* variables used for drawing the text strings. */
    int x, y;
    int x_off, y_off;

    //int string_width;
    int font_height;
  
    /* find the height of the characters drawn using this font.        */
    font_height = font_info->ascent + font_info->descent;

    x = 0;
    y = font_height;    

    /* draw black border part */
    for (x_off = 0; x_off <= 4; ++x_off)
    {
      for (y_off = 0; y_off <=4; ++y_off)
      {
        XDrawString (display, window, gc_bo, x + x_off, y + y_off,
                     text,strlen(text));
        XDrawString (display, shapemask, gc_shape, x + x_off, y + y_off,
		     text, strlen (text));
      }
    }

    /* draw white top part */
/*
    for (x_off = 1; x_off <= 2; ++x_off)
    {
      for (y_off = 1; y_off <= 2; ++y_off)
      {
        XDrawString (display, window, gc_fg, x + x_off, y + y_off,
                     text,strlen(text));
      }
    }
*/
        XDrawString (display, window, gc_fg, x + 2, y + 2,
                     text,strlen(text));

    /* flush all pending requests to the X server. */

    XShapeCombineMask (display, window, ShapeBounding, 0, 0, 
                       shapemask, ShapeSet);
    XFlush(display);      
    return 0;
  }


}

int DrawXpm 
(Display* display, Window window, GC gc, char* filename)
{
  Pixmap pixmap, shapemask;
  XpmAttributes attributes;
  XGCValues gcv;
  GC gc_clip;
  int x, y;

  gcv.graphics_exposures = False;
  gcv.function = GXcopy;
  gcv.foreground = WhitePixel (display, 0);
  gcv.subwindow_mode = IncludeInferiors;

  gc_clip = XCreateGC(display, window, GCFunction
                               |GCGraphicsExposures, &gcv);

#ifdef DEBUG
  printf ("DEBUG: reading pixmap from %s\n", filename);
#endif
  XpmReadFileToPixmap (display, window, filename, &pixmap, &shapemask,
		&attributes);

//  XFillRectangle (display, shapemask, gc_clip, 0, 0, 50, 50);


/*
  XShapeCombineMask (display, window, ShapeClip, 0, 0, shapemask,
ShapeSet);
*/
/*
  XSetClipMask (display, gc_clip, shapemask);
  XSetClipOrigin (display, gc_clip, 0, 0);
  XCopyArea (display, pixmap, window, gc_clip, 0, 0, 100, 100, 0, 0);
*/

  XShapeCombineMask (display, window, ShapeBounding, 0, 0, shapemask,
ShapeSet);
  XCopyArea (display, pixmap, window, gc_clip, 0, 0, 100, 100, 0, 0);
//  XSetWindowBackgroundPixmap (display, window, pixmap);
  /* Calculate where to move the window if you want to center it */
  x = (IMG_W - attributes.width) / 2;
  y = (IMG_H - attributes.height) / 2;

  XMoveWindow (display, window, x, y);
  XFlush(display);      
  return 1;
}

int main (int argc, char *argv[])
{
  Display* display;             /* pointer to X Display structure.           */
  Window img_w, msg_w;

  char *display_name = getenv("DISPLAY");  /* address of the X display.      */

  int screen_num;               /* number of screen to place the window on.  */
  unsigned int display_width,
               display_height;  /* height and width of the X display.        */

  GC gc, gc_bo;                 /* GC (graphics context) used fordrawing     */
                                /*  in our window.                           */
  char* font_name = DEF_FONT;   /* font to use for drawing text.   */

  cfg_t cfg;					/* save our configuration options */

#ifdef DEBUG
  printf ("DEBUG: parsing config options\n");
#endif

  parse_config (argc, argv, &cfg);

  if (argc == 1)
  {
    printf ("Please supply a line of text to display !\n");
    return 1;
  }

  /* open connection with the X server. */
  display = XOpenDisplay(display_name);
  if (display == NULL) 
  {
    fprintf(stderr, "%s: cannot connect to X server '%s'\n",
            argv[0], display_name);
    exit(1);
  }
  
  /* get the geometry of the default screen for our display. */
  screen_num = DefaultScreen (display);
  display_width = DisplayWidth (display, screen_num);
  display_height = DisplayHeight (display, screen_num);

  img_w = CreateWindow (display, screen_num, 0, 0, IMG_W, IMG_H);
  msg_w = CreateWindow (display, screen_num, 50, 0, MSG_W, MSG_H);

  /* allocate a new GC (graphics context) for drawing in the window. */

#ifdef DEBUG
  printf ("DEBUG: creating gc\n");
#endif

  gc = CreateGC (display, msg_w, cfg.color_name);
  gc_bo = CreateGC (display, msg_w, "black");
  XSync(display, False);

  /* try to load the given font. */
#ifdef DEBUG
  printf ("DEBUG: loading font\n");
#endif

  font_info = XLoadQueryFont(display, cfg.font_name);
  if (!font_info) 
  {
      fprintf(stderr, "XLoadQueryFont: failed loading font '%s'\n",font_name);
      exit(-1);
  }

  /* assign the given font to our GC. */
  XSetFont(display, gc, font_info->fid);
  XSetFont(display, gc_bo, font_info->fid);

#ifdef DEBUG
  XSync (display, False);
  printf ("DEBUG: drawing text\n");
#endif

  DrawText (display, msg_w, font_info, argv[optind + 1], gc, gc_bo);

#ifdef DEBUG
  XSync (display, False);
  printf ("DEBUG: drawing XPM\n");
#endif

  if (cfg.icon_file != NULL)
  {
    printf ("Drawing XPM from %s\n", cfg.icon_file);
    DrawXpm (display, img_w, gc_bo, cfg.icon_file);
  }


  sleep(4);

  /* close the connection to the X server. */
  XCloseDisplay(display);

  return 0;
}
